@@ -1,7 +1,8 @@
 ######################################################################
 ## These functions are directly copied from the cacheSweave package.
 ## The reason they are copied here is because they are internal functions
-## and hence are not exported into the global environment.  Some functions ## are also hacked a little, which is the reason the ::: operator is not 
+## and hence are not exported into the global environment.  Some functions 
+## are also hacked a little, which is the reason the ::: operator is not 
 ## used. The  original comments and header are preserved.
 
 
@@ -86,7 +87,7 @@ isNewOrModified <- function(specials, e1, e2) {
 ## If 'source()' was used, there may be new symbols in the global
 ## environment, unless 'source(local = TRUE)' was used.  Also applies
 ## for 'set.seed()'.
-        
+
 checkNewSymbols <- function(e1, e2) {
         if(identical(e1, e2))
                 return(character(0))
@@ -129,6 +130,8 @@ evalAndDumpToDB <- function(db, expr, exprDigest) {
         ## Dump the values of the keys to the database
         dumpToDB(db, list = keys, envir = env)
 
+	if(length(keys) > 0)
+		copy2env(keys, env, globalenv())
         keys
 }
 
@@ -140,6 +143,72 @@ mangleDigest <- function(x) {
         paste(".__", x, "__", sep = "")
 }
 
+hash <- function(object) {
+	digest(object, algo = "sha1")
+}
+
+hashExpr <- function(expr) {
+	expr <- deparse(expr, width.cutoff = 60)
+	hash(expr)
+}
+
+cacheSweaveEvalWithOpt <- function (expr, options) {
+  chunkDigest <- options$chunkDigest
+  
+  ## 'expr' is a single expression, so something like 'a <- 1'
+  res <- NULL
+
+  if(!options$eval)
+          return(res)
+  if(options$cache) {
+    cachedir <- getCacheDir()
+
+    ## Create database name from chunk label and MD5
+    ## digest
+    dbName <- makeChunkDatabaseName(cachedir, options, chunkDigest)
+    exprDigest <- mangleDigest(hashExpr(expr))
+
+    ## Create 'stashR' database
+    db <- new("localDB", dir = dbName, name = basename(dbName))
+
+    ## If the current expression is not cached, then
+    ## evaluate the expression and dump the resulting
+    ## objects to the database.  Otherwise, just read the
+    ## vector of keys from the database
+
+    if(!dbExists(db, exprDigest)) {
+
+      keys <- try({
+        evalAndDumpToDB(db, expr, exprDigest)
+      }, silent = TRUE)
+
+      ## If there was an error then just return the
+      ## condition object and let Sweave deal with it.
+      if(inherits(keys, "try-error"))
+        return(keys)
+
+    } else {
+
+      keys <- dbFetch(db, exprDigest)
+      dbLazyLoad(db, globalenv(), keys)
+
+    }
+    keys
+
+  } else {
+
+    ## If caching is turned off, just evaluate the expression
+    ## in the global environment
+    res <- try(withVisible(eval(expr, .GlobalEnv)),
+               silent=TRUE)
+    if(inherits(res, "try-error"))
+      return(res)
+    if(options$print | (options$term & res$visible))
+      print(res$value)
+  }
+  res
+}
+
 makeMapFileName <- function(Rnwfile) {
         mapfile <- sub("\\.Rnw$", "\\.map", Rnwfile)
 
