@@ -42,7 +42,7 @@ pgfSweaveSetup <- function(file, syntax,
               eval=TRUE, split=FALSE, stylepath=TRUE,
               pdf=FALSE, eps=FALSE, cache=FALSE, pgf=FALSE,
               tikz=TRUE, external=FALSE, sanitize = FALSE,
-              highlight = TRUE, tidy = FALSE, tex.driver="pdflatex")
+              highlight = TRUE, tidy = FALSE)
 {
     out <- utils::RweaveLatexSetup(file, syntax, output=output, quiet=quiet,
                      debug=debug, echo=echo, eval=eval,
@@ -58,10 +58,11 @@ pgfSweaveSetup <- function(file, syntax,
     out$options[["pgf"]] <- pgf
     out$options[["tikz"]] <- tikz
     out$options[["external"]] <- external
-    out$options[["tex.driver"]] <- tex.driver
     out$options[["sanitize"]] <- sanitize
     out$options[["highlight"]] <- ifelse(echo,highlight,FALSE)
     out$options[["tidy"]] <- tidy
+    out$options[["relwidth"]] <- 1
+    out$options[["relheight"]] <- 1
     out[["haveHighlightSyntaxDef"]] <- FALSE
     out[["haveRealjobname"]] <- FALSE
     ## end [CWB]
@@ -72,80 +73,46 @@ pgfSweaveSetup <- function(file, syntax,
     ## End additions [RDP]
 
     ## [CWB]  create a shell script with a command for each modified graphic
-    out[["shellFile"]] <- makeExternalShellScriptName(file)
-    out[["srcfileName"]] <- sub("\\.[Rr]nw$", "\\.tex", file)
-    out[["jobname"]] <- basename(removeExt(file))
-    file.create(out[["shellFile"]])  ## Overwrite an existing file
+    out[["srcfileName"]] <-paste(tools::file_path_sans_ext(file), "tex", sep='')
+    out[["jobname"]] <- basename(tools::file_path_sans_ext(file))
     ######################################################################
-
-      # Avoids creation of Rplots.pdf
-    options(device = function(...) {
-        .Call("R_GD_nullDevice", PACKAGE = "grDevices")
-    })
-
+    
     out
 }
 
-    # changes in Sweave in 2.12 make it necessary to copy this function here
-    # to register the tex.driver option, I wish there was an easier way....
+    # This function checks the options. Most of the checks is delegated to
+    # utils::RweaveLatexOptions, expect for the rel* options
 pgfSweaveOptions <- function(options)
 {
-
-    ## ATTENTION: Changes in this function have to be reflected in the
-    ## defaults in the init function!
-
-    ## convert a character string to logical
-    c2l <- function(x){
-        if(is.null(x)) return(FALSE)
-        else return(as.logical(toupper(as.character(x))))
-    }
-
-    NUMOPTS <- c("width", "height")
-    NOLOGOPTS <- c(NUMOPTS, "results", "prefix.string",
-                   "engine", "label", "strip.white",
-                   "pdf.version", "pdf.encoding", "tex.driver")
-    for(opt in names(options)){
-        if(! (opt %in% NOLOGOPTS)){
-            oldval <- options[[opt]]
-            if(!is.logical(options[[opt]])){
-                options[[opt]] <- c2l(options[[opt]])
-            }
-            if(is.na(options[[opt]]))
-                stop(gettextf("invalid value for '%s' : %s", opt, oldval),
-                     domain = NA)
-        }
-        else if(opt %in% NUMOPTS){
-            options[[opt]] <- as.numeric(options[[opt]])
+    options.norel <- options
+    options.norel[c("relwidth","relheight")] <- NULL
+    options.norel <- utils::RweaveLatexOptions(options.norel)
+    for(opt in c("relwidth","relheight")) {
+        if(!is.null(options[[opt]])) {
+            options.norel[opt] <- as.numeric(options[[opt]])
         }
     }
-
-    if(!is.null(options$results))
-        options$results <- tolower(as.character(options$results))
-    options$results <- match.arg(options$results,
-                                 c("verbatim", "tex", "hide"))
-
-    if(!is.null(options$strip.white))
-        options$strip.white <- tolower(as.character(options$strip.white))
-    options$strip.white <- match.arg(options$strip.white,
-                                     c("true", "false", "all"))
-    options
+    options.norel$width <- options.norel$width * options.norel$relwidth
+    options.norel$height <- options.norel$height * options.norel$relheight
+    options.norel
 }
 
-    # Copied from Sweave in R version 2.12, internal chages make it necessary
-    # to register the tex.driver option as a "NOLOGOPT"
     # This function checks for the \usepackage{Sweave} line and the
-    # \pgfrealjobname line
+    # \tikzexternalize line
 pgfSweaveWritedoc <- function(object, chunk)
 {
     linesout <- attr(chunk, "srclines")
 
     if(length(grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)))
         object$havesty <- TRUE
-
-    haverealjobname <- FALSE
-    if(length(grep("\\pgfrealjobname\\{.*\\}", chunk)))
-        haverealjobname <- TRUE
-
+        
+    if(is.null(object$havetikzexternalize)){
+        object$havetikzexternalize <- 
+        if(length(grep("\\tikzexternalize.*^", chunk)))
+            TRUE
+        else 
+            FALSE
+    }
 
     if(!object$havesty){
       begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
@@ -159,54 +126,45 @@ pgfSweaveWritedoc <- function(object, chunk)
         }
     }
 
-      # add pgfrealjobname if it doesnt exist
-    if(!haverealjobname){
-     begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
-     which <- grep(begindoc, chunk)
-      # if Sweave line also does not exist
-     #otherwhich <- grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)
-     otherwhich <- grep("^[[:space:]]*\\\\usepackage\\{.*tikz.*\\}", chunk)
-     other <- FALSE
-     if(length(which) == 0){which <- otherwhich;other <- T}
-     if (length(which)) {
-     if(length(which) > 1) {cat('haverealjobname',which,'\n');browser()}
-      chunk[which] <- 
-        if(other) 
-          paste(chunk[which],"\n\\pgfrealjobname{",object$jobname,"}\n", sep="")
-        else
-          paste("\\pgfrealjobname{",object$jobname,"}\n",chunk[which], sep="")
-      linesout <- linesout[c(1L:which, which, seq(from=which+1L, length.out=length(linesout)-which))]
-      object$haverealjobname <- TRUE
+      # add tikzexternalize if it doesnt exist
+    if(!object$havetikzexternalize){
+        # add the lines after the \usepackage{tikz} statement
+      which <- grep("^[[:space:]]*\\\\usepackage\\{.*tikz.*\\}", chunk)
+      if(length(which)){
+        chunk[which] <- paste(chunk[which],"\\usetikzlibrary{external}",
+          "\\tikzexternalize[mode=list and make]\n", sep="\n")
+        linesout <- linesout[c(1L:which, which, seq(from=which+1L, length.out=length(linesout)-which))]
+        object$havetikzexternalize <- TRUE
       }
     }
      
-     # always add the syntax definitions because there is no real way to
-     # check if a single code chunk as the option before hand.
-      if (!object$haveHighlightSyntaxDef){
-              # get the latex style definitions from the highlight package
-        tf <- tempfile()
-        cat(styler('default', 'sty', styler_assistant_latex),sep='\n',file=tf)
-        cat(boxes_latex(),sep='\n',file=tf,append=T)
-        hstyle <- readLines(tf)
-          # find where to put the style definitions
-        begindoc <- "^[[:space:]]*\\\\begin\\{document\\}"
-          # first try to put it before the begin{document} command
-          # this may not work if there is a code chunk before it
-        which <- grep(begindoc, chunk)
-          # also try to put it by the Sweave style file, 
-        otherwhich <- grep("\\usepackage[^\\}]*Sweave.*\\}", chunk)
-        if(!length(which)) which <- otherwhich
-        
-              # put in the style definitions before the \begin{document}
-              # or \usepackage{Sweave}
-        if(length(which)) {
-    if(length(which) > 1) cat('havehighlight',which,'\n')          
-          chunk <- c(chunk[1:(which-1)],hstyle,chunk[which:length(chunk)])
-          linesout <- linesout[c(1L:which, which, seq(from = which +
-              1L, length.out = length(linesout) - which))]
-          object$haveHighlightSyntaxDef <- TRUE
-        }
+      # always add the syntax definitions because there is no real way to
+      # check if a single code chunk as the option before hand.
+    if (!object$haveHighlightSyntaxDef){
+      
+        # get the latex style definitions from the highlight package
+      tf <- tempfile()
+      cat(styler('default', 'sty', styler_assistant_latex),sep='\n',file=tf)
+      cat(boxes_latex(),sep='\n',file=tf,append=T)
+      hstyle <- readLines(tf)
+
+
+        # find where to put the style definitions
+      begindoc <- "^[[:space:]]*\\\\documentclass.*$"
+      which <- grep(begindoc, chunk)
+
+        # add definitions for highlight environment
+      hstyle <- c(hstyle, "\\newenvironment{Houtput}{\\raggedright}{%\n%\n}")
+
+            # put in the style definitions after the \documentclass command
+      if(length(which)) {
+        chunk <- c(chunk[1:which],hstyle,chunk[(which+1):length(chunk)])
+        linesout <- linesout[c(1L:which, which, seq(from = which +
+            1L, length.out = length(linesout) - which))]
+        object$haveHighlightSyntaxDef <- TRUE
       }
+    }
+    
     while(length(pos <- grep(object$syntax$docexpr, chunk)))
     {
         cmdloc <- regexpr(object$syntax$docexpr, chunk[pos[1L]])
@@ -223,6 +181,7 @@ pgfSweaveWritedoc <- function(object, chunk)
 
         chunk[pos[1L]] <- sub(object$syntax$docexpr, val, chunk[pos[1L]])
     }
+    
     while(length(pos <- grep(object$syntax$docopt, chunk)))
     {
         opts <- sub(paste(".*", object$syntax$docopt, ".*", sep=""),
@@ -312,13 +271,6 @@ pgfSweaveRuncode <- function(object, chunk, options) {
   SweaveHooks(options, run=TRUE)
 
     # remove unwanted "#line" directives added by R 2.12
-  removeLineJunk <- function(chunk){
-    lines <- grep("#line", chunk)
-    srclines <- attr(chunk, "srclines")
-    chunk <- chunk[-lines]
-    attr(chunk, "srclines") <- srclines[-lines]
-    chunk
-  }
   chunk <- removeLineJunk(chunk)
 
   ## parse entire chunk block
@@ -331,17 +283,6 @@ pgfSweaveRuncode <- function(object, chunk, options) {
 
   RweaveTryStop(chunkexps, options)
 
-  ## [CWB] Create a DB entry which is simply the digest of the text of
-  ## the chunk so that if anything has changed the chunk will be
-  ## recognised as having a change if ANYTHING gets changed, even white
-  ## space. this still doesnt fix the problem of actually caching the
-  ## plotting commands and others which do not create objects in the
-  ## global environment when they are evaluated but at least the figures
-  ## will get regenerated.
-  chunkTextEvalString <- paste("chunkText <- '",
-    digest(paste(chunk,collapse='')), "'", sep='')
-  attr(chunk, "digest") <- digest(paste(chunk,collapse=''))
-  ## end [CWB]
 
   ## Adding my own stuff here [RDP]
   ## Add 'chunkDigest' to 'options'
@@ -350,6 +291,8 @@ pgfSweaveRuncode <- function(object, chunk, options) {
 
   openSinput <- FALSE
   openSchunk <- FALSE
+    # for the highlighted output environment
+  openHoutput <- FALSE
 
   if(length(chunkexps)==0)
     return(object)
@@ -419,6 +362,13 @@ pgfSweaveRuncode <- function(object, chunk, options) {
         }
           cat("\\begin{Sinput}",file=chunkout, append=TRUE)
           openSinput <- TRUE
+      }else if(!openHoutput & options$highlight){
+
+          cat("\\begin{Houtput}\n",file=chunkout, append=TRUE)
+          linesout[thisline + 1] <- srcline
+          thisline <- thisline + 1
+          openHoutput <- TRUE
+
       }
 
          # Actual printing of chunk code
@@ -436,12 +386,13 @@ pgfSweaveRuncode <- function(object, chunk, options) {
 
           if(nce == 1)
             cat(newline_latex(),file=chunkout, append=TRUE)
+
           highlight(parser.output=parser(text=dce),
             renderer=renderer_latex(document=FALSE),
             output = chunkout, showPrompts=TRUE,final.newline = TRUE)
               # highlight doesnt put in an ending newline for some reason
           cat(newline_latex(),file=chunkout, append=TRUE)
-          
+
         }
 
       }else{
@@ -464,18 +415,20 @@ pgfSweaveRuncode <- function(object, chunk, options) {
         ## tmpcon <- textConnection("output", "w")
         ## avoid the limitations (and overhead) of output text connections
       tmpcon <- file()
+        ## collect output to temporary file
       sink(file=tmpcon)
       err <- NULL
 
+
+        # use the cacheSweave mechanism to cache objects
       if(options$eval) err <- cacheSweaveEvalWithOpt(ce, options)
-      ## [RDP] end change
 
       cat("\n") # make sure final line is complete
       sink()
       output <- readLines(tmpcon)
       close(tmpcon)
 
-      ## delete empty output
+        ## delete empty output
       if(length(output)==1 & output[1]=="") output <- NULL
 
       RweaveTryStop(err, options)
@@ -546,6 +499,11 @@ pgfSweaveRuncode <- function(object, chunk, options) {
     thisline <- thisline + 1
   }
 
+  if(openHoutput){
+    cat("\\end{Houtput}\n", file=chunkout, append=TRUE)
+    linesout[thisline + 1] <- srcline
+    thisline <- thisline + 1
+  }
 
   if(is.null(options$label) & options$split)
     close(chunkout)
@@ -558,13 +516,18 @@ pgfSweaveRuncode <- function(object, chunk, options) {
   }
 
   if(options$fig && options$eval){
-
-    ## [CWB] adding checking for external options
-    pdfExists <- file.exists(paste(chunkprefix, "pdf", sep="."))
-    epsExists <- file.exists(paste(chunkprefix, "eps", sep="."))
+      
+    chunkChanged <- 
+      if( options$external )
+        hasChunkChanged(chunk,chunkprefix,options)
+      else
+        TRUE
+    if(chunkChanged & options$external) cat('(Re)Running External Chunk:',options$label,'\n')
 
     if(options$eps & !options$pgf & !options$tikz){
-      if(!options$external | !epsExists ){
+        # Still apply graphics caching to postscript device
+        # only regenerate the image if the chunk has been changed.
+      if(!options$external | chunkChanged ){
 
           # [CWB] the useKerning option was added in R 2.9.0
           # so check for the R version and set the
@@ -588,7 +551,9 @@ pgfSweaveRuncode <- function(object, chunk, options) {
       }
     }
     if(options$pdf & !options$pgf & !options$tikz){
-      if(!options$external | !pdfExists ){
+      # Still apply graphics caching to pdf device
+      # only regenerate the image if the chunk has been changed.
+      if(!options$external | chunkChanged ){
         grDevices::pdf(file=paste(chunkprefix, "pdf", sep="."),
           width=options$width, height=options$height,
           version=options$pdf.version, encoding=options$pdf.encoding)
@@ -614,7 +579,7 @@ pgfSweaveRuncode <- function(object, chunk, options) {
     ##
 
     if(options$pgf){
-      if( !pdfExists ){
+      if( !options$external | chunkChanged ){
 
           # [CWB] the useKerning option was added in R 2.9.0
           # so check for the R version and set the
@@ -646,8 +611,9 @@ pgfSweaveRuncode <- function(object, chunk, options) {
         if(inherits(err, "try-error")) stop(err)
       }
     }
-    if(options$tikz){
-      if(ifelse(options$tex.driver == "latex", !epsExists, !pdfExists)){
+
+    if( options$tikz ){
+      if( !options$external | chunkChanged ){
         tikzDevice::tikz(file=paste(chunkprefix, "tikz", sep="."),
           width=options$width, height=options$height,
           sanitize=options$sanitize)
@@ -658,58 +624,55 @@ pgfSweaveRuncode <- function(object, chunk, options) {
           })
         grDevices::dev.off()
         if(inherits(err, "try-error")) stop(err)
+        
       }
     }
-    
-      # External option, write commands to externalize graphics if the 
-      # graphic file does not exist
-    if( options$external ){
-      if(ifelse(options$tex.driver == "latex", !epsExists, !pdfExists) && 
-         (!options$pdf && !options$eps)){
 
-        shellFile <- object[["shellFile"]]
-        tex.driver <- options[["tex.driver"]]
-
-        cat(tex.driver, ' --jobname=', chunkprefix,' ',
-          object[["srcfileName"]], '\n', sep='',  file=shellFile, append=TRUE)
-        if(tex.driver == "latex"){
-          cat('dvipdf ',paste(chunkprefix,'dvi',sep='.'), '\n', sep='',
-            file=shellFile, append=TRUE)
-          cat('pdftops -eps ',paste(chunkprefix,'pdf',sep='.'),
-            '\n', sep='', file=shellFile, append=TRUE)
-        }
-      }
-    }
-      # Write the extrnalization commands if the user does not want 
-      # to do it themselves already
+      # Write the extrnalization commands
     if(options$include && options$external) {
-      cat("\n\\beginpgfgraphicnamed{",chunkprefix,"}\n",sep="",
+      cat("\n\\tikzsetnextfilename{",chunkprefix,"}\n",sep="",
         file=object$output, append=TRUE)
+      cat("\n\\tikzexternalfiledependsonfile{",chunkprefix,"}{",
+        paste(chunkprefix, "tikz", sep="."),"}\n",sep="",
+        file=object$output, append=TRUE)
+        
       linesout[thisline + 1] <- srcline
       thisline <- thisline + 1
+      
     }
+
       # Write the includegraphics command for eps or pdf 
       # only if we are not useing pgf or tikz
     if(options$include && !options$pgf && !options$tikz && !options$external) {
+
       cat("\\includegraphics{", chunkprefix, "}\n", sep="",
         file=object$output, append=TRUE)
       linesout[thisline + 1] <- srcline
       thisline <- thisline + 1
+
     }
       # input statements for tikz and pgf
     if(options$include && (options$pgf || options$tikz)) {
-      #if tikz takes precident over pgf option
+
+        #if tikz takes precident over pgf option
       suffix <- ifelse(options$tikz,'tikz','pgf')
+      if(!options$external){
+        cat("{\\tikzexternaldisable\n", sep="", file=object$output, append=TRUE)
+        linesout[thisline + 1] <- srcline
+        thisline <- thisline + 1
+      }
+      
       cat("\\input{", paste(chunkprefix,suffix,sep='.'),
         "}\n", sep="", file=object$output, append=TRUE)
       linesout[thisline + 1] <- srcline
       thisline <- thisline + 1
-    }
-      # ending externalization command
-    if(options$include && options$external) {
-      cat("\\endpgfgraphicnamed\n",sep="",file=object$output, append=TRUE)
-      linesout[thisline + 1] <- srcline
-      thisline <- thisline + 1
+      
+      if(!options$external){
+        cat("}\n", sep="", file=object$output, append=TRUE)
+        linesout[thisline + 1] <- srcline
+        thisline <- thisline + 1
+      }
+
     }
     ##end graphics options [CWB]
     #############################################
